import string

class HangmanSolver:
    
    def __init__(self, pattern, initialGuesses):
        self.vowels = 'aeiou'
        self.pattern = pattern
        self.possWords = []
        self.guesses = initialGuesses
        self.noVowelsBeyond = max([pattern.find(vowel) for vowel in self.vowels])
        for char in pattern:
            if char != '-' and char not in self.guesses and char not in initialGuesses:
                self.guesses += char
    
    def findWords(self):
        f = open('words.txt', 'r')
        line = f.readline()
        while (line != ''):
            word = line.split()[0].lower()
            if len(word) > len(self.pattern):
                break
            elif len(word) == len(self.pattern):
                i = 0
                while i < len(word):
                    noMoreVowels = i > self.noVowelsBeyond
                    if self.pattern[i] == '-':
                        if word[i] in self.guesses:
                            # the letter was already guessed
                            break
                        if noMoreVowels and word[i] in self.vowels:
                            # not supposed to be any more vowels
                            break
                    else:
                        if self.pattern[i] != word[i]:
                            break
                    i += 1
                if i == len(word):
                    print word
                    self.possWords.append(word)
            line = f.readline()
        f.close()
        return self.possWords
    
    def updateWords(self):
        for word in self.possWords[:]:
            i = 0
            while i < len(word):
                if (self.pattern[i] == '-' and word[i] in self.guesses):
                    self.possWords.remove(word)
                    break
                if (self.pattern[i] != '-' and self.pattern[i] != word[i]):
                    self.possWords.remove(word)
                    break
                i += 1
    
    def suggestGuesses(self):
        letterHits = dict((char, 0) for char in string.lowercase if char not in self.guesses)
        for word in self.possWords:
            seen = self.guesses
            for char in word:
                if char not in seen:
                    letterHits[char] += 1
                    seen += char
        numWords = float(len(self.possWords))
        maxNum = max(letterHits.values())
        if maxNum == 0:
            return None
        suggested = {}
        for char in letterHits.keys():
            if letterHits[char] == maxNum:
                suggested[char] = int(maxNum/numWords*100)
        return suggested
    
    def guess(self, guess, newPattern=''):
        if len(guess) == 1 and guess not in self.guesses:
            self.guesses += guess
            # guessed wrong
            if newPattern == '':
                self.possWords = [word for word in self.possWords[:] if guess not in word]
            else:
                self.pattern = newPattern
                self.updateWords()
            return self.possWords

puzzle = raw_input('Enter the puzzle (with "-" for unknown letters): ')
guessed = raw_input('Any additional letters guessed so far (no separation): ')
solver = HangmanSolver(puzzle, guessed)
possWords = solver.findWords()
while (True):
    guesses = solver.suggestGuesses()
    if guesses == None:
        print 'This puzzle is impossible (or you won).'
        break
    print len(possWords), 'words possible'
    print 'Suggested guesses: '
    for guess, prob in guesses.items():
        print guess, ':', str(prob) + '%'
    guess = raw_input('What is your guess? ')
    newPattern = raw_input('If ' + guess + ' was in the puzzle, new word (or just leave empty): ')
    possWords = solver.guess(guess, newPattern)
    for word in possWords:
        print word
    